@@ -119,7 +119,7 @@ function HistoryListModel(parent) {
         data.append("nzbfile", $(form.nzbFile)[0].files[0]);
         data.append("job", $('#modal-retry-job input[name="retry_job_id"]').val());
         data.append("password", $('#retry_job_password').val());
-        data.append("session", apiKey);
+        data.append("apikey", apiKey);
 
         // Add
         $.ajax({
@@ -169,7 +169,6 @@ function HistoryListModel(parent) {
 
     // Toggle showing failed
     self.toggleShowFailed = function(data, event) {
-
         // Set the loader so it doesn't flicker and then switch
         self.isLoading(true)
         self.showFailed(!self.showFailed())
@@ -177,7 +176,20 @@ function HistoryListModel(parent) {
         $('#history-options a').tooltip('hide')
         // Force refresh
         self.parent.refresh(true)
+    }
 
+    // Retry all failed
+    self.retryAllFailed = function(data, event) {
+        // Ask to be sure
+        if(confirm(glitterTranslate.retryAll)) {
+            // Send the command
+            callAPI({
+                mode: 'retry_all'
+            }).then(function() {
+                // Force refresh
+                self.parent.refresh(true)
+            })
+        }
     }
 
     // Empty history options
@@ -296,7 +308,15 @@ function HistoryModel(parent, data) {
     // Processing or done?
     self.processingDownload = ko.pureComputed(function() {
         var status = self.status();
-        return(status === 'Extracting' || status === 'Moving' || status === 'Verifying' || status === 'Running' || status == 'Repairing')
+        // When we can cancel
+        if (status === 'Extracting' || status === 'Verifying' || status == 'Repairing' || status === 'Running') {
+            return 2
+        }
+        // These cannot be cancelled
+        if(status === 'Moving') {
+            return 1
+        }
+        return false;
     })
 
     // Format status text
@@ -313,23 +333,21 @@ function HistoryModel(parent, data) {
         return self.script_line();
     });
 
-    // Extra history column
-    self.extraText = ko.pureComputed(function() {
+    // Extra history columns
+    self.showColumn = function(param) {
         // Picked anything?
-        switch(self.parent.parent.extraHistoryColumn()) {
+        switch(param) {
             case 'speed':
                 // Anything to calculate?
                 if(self.historyStatus.bytes() > 0 && self.historyStatus.download_time() > 0) {
-                    var theSpeed = self.historyStatus.bytes()/self.historyStatus.download_time();
-                    theSpeed = theSpeed/1024;
-
-                    // MB/s or KB/s
-                    if(theSpeed > 1024) {
-                        theSpeed = theSpeed/1024;
-                        return theSpeed.toFixed(1) + ' MB/s'
-                    } else {
-                        return Math.round(theSpeed) + ' KB/s'
-                    }
+                    try {
+                        // Extract the Download section
+                        var downloadLog = ko.utils.arrayFirst(self.historyStatus.stage_log(), function(item) {
+                            return item.name() == 'Download'
+                        });
+                        // Extract the speed
+                        return downloadLog.actions()[0].match(/(\S*\s\S+)(?=<br\/>)/)[0]
+                    } catch(err) { }
                 }
                 return;
             case 'category':
@@ -341,17 +359,24 @@ function HistoryModel(parent, data) {
                 return self.historyStatus.size();
         }
         return;
-    })
+    };
 
     // Format completion time
     self.completedOn = ko.pureComputed(function() {
         return displayDateTime(self.completed(), parent.parent.dateFormat(), 'X')
     });
 
+    // Subscribe to retryEvent so we can load the password
+    self.canRetry.subscribe(function() {
+        self.updateAllHistory = true;
+    })
+
     // Re-try button
     self.retry = function() {
         // Set JOB-id
         $('#modal-retry-job input[name="retry_job_id"]').val(self.nzo_id)
+        // Set password
+        $('#retry_job_password').val(self.historyStatus.password())
         // Open modal
         $('#modal-retry-job').modal("show")
     };
@@ -373,7 +398,10 @@ function HistoryModel(parent, data) {
     self.addHistoryStatusStuff = function(item) {
         $(item).find('.history-status-modallink a').click(function(e) {
             // Modal or 'More' click?
-            if($(this).is('.history-status-more')) {
+            if($(this).is('.history-status-dmca')) {
+                // Pass
+                return true;
+            } else if($(this).is('.history-status-more')) {
                 // Expand the rest of the text and hide the button
                 $(this).siblings('.history-status-hidden').slideDown()
                 $(this).hide()
@@ -392,27 +420,32 @@ function HistoryModel(parent, data) {
 
     // Delete button
     self.deleteSlot = function(item, event) {
-        // Are we not still processing?
-        if(item.processingDownload() || item.processingWaiting()) return false;
-
         // Confirm?
-        if(!self.parent.parent.confirmDeleteHistory() || confirm(glitterTranslate.removeDow1)) {
-            callAPI({
-                mode: 'history',
-                name: 'delete',
-                del_files: 1,
-                value: self.nzo_id
-            }).then(function(response) {
-                if(response.status) {
-                    // Fade and remove
-                    $(event.currentTarget).parent().parent().fadeOut(fadeOnDeleteDuration, function() {
+        if(!self.parent.parent.confirmDeleteHistory() || confirm(glitterTranslate.deleteMsg + ":\n" + item.historyStatus.name() + "\n\n" + glitterTranslate.removeDow1)) {
+            // Are we still processing and it can be stopped?
+            if(item.processingDownload() == 2) {
+                callAPI({
+                    mode: 'cancel_pp',
+                    value: self.nzo_id
+                })
+                // All we can do is wait
+            } else {
+                // Delete the item
+                callAPI({
+                    mode: 'history',
+                    name: 'delete',
+                    del_files: 1,
+                    value: self.nzo_id
+                }).then(function(response) {
+                    if(response.status) {
                         // Make sure no flickering (if there are more items left) and then remove
                         self.parent.isLoading(self.parent.totalItems() > 1)
                         self.parent.historyItems.remove(self);
                         self.parent.parent.refresh();
-                    })
-                }
-            });
+                    }
+                });
+            }
+
         }
     };
 
