@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,22 +25,6 @@ from sabnzbd.newsunpack import *
 
 
 class TestNewsUnpack:
-    @pytest.mark.parametrize(
-        "test_input, expected_output",
-        [
-            (["cmd1", 9, "cmd3"], '"cmd1" "9" "cmd3"'),  # sending all commands as valid string
-            (["", "cmd1", "5"], '"" "cmd1" "5"'),  # sending blank string
-            (["cmd1", None, "cmd3", "tail -f"], '"cmd1" "" "cmd3" "tail -f"'),  # sending None in command
-            (["cmd1", 0, "ps ux"], '"cmd1" "" "ps ux"'),  # sending 0
-        ],
-    )
-    def test_list_to_cmd(self, test_input, expected_output):
-        """ Test to convert list to a cmd.exe-compatible command string """
-
-        res = list2cmdline(test_input)
-        # Make sure the output is cmd.exe-compatible
-        assert res == expected_output
-
     def test_is_sfv_file(self):
         assert is_sfv_file("tests/data/good_sfv_unicode.sfv")
         assert is_sfv_file("tests/data/one_line.sfv")
